'use strict';

// ---------------------------------------------------------
// "Classes" that represent AST nodes
// ---------------------------------------------------------

class Prog extends AST {
  constructor(es) {
    super();
    this.es = es;
  }
  eval(table) {
	console.log("table is " + table);
	const results = this.es.map(e => e.eval(table));
  	return results[results.length-1];
  }
}

/* TODO: define eval(..) for the classes where it's not yet defined */
class Num extends AST {
  constructor(n) {
    super();
    this.n = n;
  }
  eval(table) {
  	return this.n;
  }
}

class Add extends AST {
  constructor(e1, e2) {
    super();
    this.e1 = e1;
    this.e2 = e2;
  }
  eval(table) {
  	return this.e1.eval(table) + this.e2.eval(table);
  }
}

class Sub extends AST {
  constructor(e1, e2) {
    super();
    this.e1 = e1;
    this.e2 = e2;
  }
}

class Mul extends AST {
  constructor(e1, e2) {
    super();
    this.e1 = e1;
    this.e2 = e2;
  }
}

class Div extends AST {
  constructor(e1, e2) {
    super();
    this.e1 = e1;
    this.e2 = e2;
  }
}

class Pow extends AST {
  constructor(e1, e2) {
    super();
    this.e1 = e1;
    this.e2 = e2;
  }
}

class Ref extends AST {
  constructor(x) {
    super();
    this.x = x;
  }
}

class Assign extends AST {
  constructor(x, e) {
    super();
    this.x = x;
    this.e = e;
  }
  eval(table) {
  	table[this.x] = this.e.eval(table);
	// TODO: what should this method return?
  }
}

